# -*- coding: utf-8 -*-

import json

from odoo.addons.saas_worker.compat import http, request, route
from odoo.addons.saas_worker.util import from_role


class Internal(http.Controller):

    @from_role("private", signed=False)
    @route("/_internal/outdated_users", auth="none", csrf=False, readonly=True)
    def outdated_users(self, **kw):
        """
        Return the (possibly) outdated internal users. A user is considered outdated if:
        - the record is archived
        - the write_date is < 24 hours

        In case a user was modified recently, the session token is computed with an hardcoded
        `sid` and returned in the result `dict`. The client if then responsible for comparing
        the session token provided.

        return dict: the outdated user ids with the structure:
                     {
                         "inactive": [<ids>],
                         "sess_change": [(<id>, <session_token>)],
                     }
        """

        res = {}
        Users = request.env["res.users"].sudo()

        # Inactive users
        request.env.cr.execute(
            """
            SELECT id
              FROM res_users
             WHERE share != 't'
               AND active = 'f'
            """
        )
        res["inactive"] = [r[0] for r in request.env.cr.fetchall()]

        # Session changed
        request.env.cr.execute(
            """
            SELECT id
              FROM res_users
             WHERE share != 't'
               AND active = 't'
               AND write_date > now() AT TIME ZONE 'UTC' - interval '1 day'
            """
        )
        res["sess_change"] = [
            (r[0], Users.browse([r[0]])._compute_session_token(str(r[0])))
            for r in request.env.cr.fetchall()
        ]

        return json.dumps(res)

    @from_role("act", "apps", "backend", "dns", "fin", "grafana", "iap", "ops", "sec", "tx", signed=False)
    @route("/_internal/archived_users", auth="none", csrf=False)
    def archived_users(self, **kw):
        """
        Similar to /_internal/outdated_users but without the session token part.
        It returns logins instead of ids for compatibility with other services.
        """
        res = {}

        # Inactive users
        request.env.cr.execute(
            """
            SELECT login
              FROM res_users
             WHERE share != 't'
               AND active = 'f'
            """
        )
        res["inactive"] = [r[0] for r in request.env.cr.fetchall()]

        return json.dumps(res)
